@@ -17,10 +17,8 @@ from sys import float_info
 import numpy as np
 import functools
 from scipy.optimize import minimize
-from scipy.special import factorial
-from scipy.special import logsumexp
-from scipy.stats import poisson
 from mpmath import polylog
+import midr.log as log
 
 import c_archimedean as c_arch
 
@@ -72,7 +70,7 @@ def lsum(x_values, is_log=True, axis=0):
     :param is_log:
     :param axis:
     :return:
-    >>> np.exp(lsum(np.array([1, 2, 3, 4]), is_log=False))
+    >>> np.exp(lsum(np.array([1, 2, 3, 4]), is_log=False, axis=0))
     array([10.])
     >>> lsum(np.array([[0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
@@ -84,6 +82,7 @@ def lsum(x_values, is_log=True, axis=0):
     ...    [0.28607729, 0.82713755, 0.7686878],
     ...    [0.22437343, 0.16907646, 0.5740400],
     ...    [0.66752741, 0.69487362, 0.3329266]]),
+    ... is_log=True,
     ... axis=0
     ... )
     array([2.8032607 , 2.71855056, 2.87969188])
@@ -97,6 +96,7 @@ def lsum(x_values, is_log=True, axis=0):
     ...    [0.28607729, 0.82713755, 0.7686878],
     ...    [0.22437343, 0.16907646, 0.5740400],
     ...    [0.66752741, 0.69487362, 0.3329266]]),
+    ... is_log=True,
     ... axis=1)
     array([1.67247612, 1.32058311, 1.69157834, 1.50086587, 1.479448  ,
            1.62823149, 1.73370009, 1.75351973, 1.43774153, 1.67670621])
@@ -129,7 +129,7 @@ def lssum(x_values, x_sign=np.nan, is_log=True):
     array([1.0729724 , 0.48860043, 0.13450368, 0.28073852, 0.85281759,
            0.77384662, 1.1716088 , 0.18345907, 0.6112606 , 0.29341583])
     """
-    return c_arch.llsum(x_values, signff(1/3.2, np.array([1, 2, 3]), 3), True)
+    return c_arch.lssum(x_values, signff(1/3.2, np.array([1, 2, 3]), 3), True)
 
 
 def signff(alpha, j, d):
@@ -161,6 +161,15 @@ def log1mexpunit(x):
     -0.14541345786885906
     """
     return c_arch.log1mexp(x);
+
+def log1mexpunit_py(x):
+    """
+    compute log(1-exp(-a)
+    :param x:
+    :return:
+    >>> log1mexpunit(2.0)
+    -0.14541345786885906
+    """
     if x <= np.log(2.0):
         return np.log(-np.expm1(-x))
     else:
@@ -176,6 +185,15 @@ def log1mexpvec(x):
     array([-0.14541346, -0.05106918, -0.01848545])
     """
     return np.asarray(c_arch.log1mexpvec(x))
+
+def log1mexpvec_py(x):
+    """
+    compute log(1-exp(-a)
+    :param x:
+    :return:
+    >>> log1mexpvec(np.array([2.0, 3.0, 4.0]))
+    array([-0.14541346, -0.05106918, -0.01848545])
+    """
     res = np.empty_like(x)
     eps = np.log(2.0)
     with np.errstate(invalid='ignore'):
@@ -196,6 +214,17 @@ def log1mexp(x):
     return log1mexpunit(x)
 
 
+def log1mexp_py(x):
+    """
+    compute log(1-exp(-a)
+    :param x:
+    :return:
+    """
+    if hasattr(x, "__len__"):
+        return log1mexpvec_py(x)
+    return log1mexpunit_py(x)
+
+
 def log1pexp(x):
     """
     compute log(1 + exp(x))
@@ -277,12 +306,12 @@ def dmle_copula_clayton(u_values):
     ...    [0.46365886, 0.2459    , 0.83277053]
     ...    ])
     ... )
-    0.2740635891728625
+    0.27406356689539874
     """
     return max_diag_pdf(
         u_values=u_values,
         diag_pdf=diag_pdf_clayton,
-        init=0.5,
+        init=10,
         bounds=(float_info.min, 1000.0)
     )
 
@@ -305,12 +334,12 @@ def dmle_copula_frank(u_values):
     ...    [0.46365886, 0.2459    , 0.83277053]
     ...    ])
     ... )
-    3.073701656631533
+    3.0737014529458255
     """
     return max_diag_pdf(
         u_values=u_values,
         diag_pdf=diag_pdf_frank,
-        init=0.5,
+        init=15,
         bounds=(float_info.min, 745.0)
     )
 
@@ -519,6 +548,7 @@ def cdf_clayton(u_values, theta, is_log=False):
         is_log=is_log
     )
 
+
 def pdf_clayton(u_values, theta, is_log=False):
     """
     compute clayton copula pdf
@@ -575,31 +605,6 @@ def pdf_clayton(u_values, theta, is_log=False):
            1.0480272 , 0.89668514, 1.00407535, 1.007351  , 1.03652896])
     """
     return c_arch.pdf_clayton(u_values, theta, is_log)
-    d = float(u_values.shape[1])
-    lu = np.sum(np.log(u_values), axis=1)
-    t_var = np.sum(ipsi_clayton(u_values, theta), axis=1)
-    res = np.zeros(shape=u_values.shape[0])
-    if theta == 0.0:
-        return res
-    if theta < 0.0:
-        for i in range(u_values.shape[0]):
-            if t_var[i] < 1.0:
-                res[i] = np.log1p(theta)
-                res[i] -= (1.0 + theta) * lu[i]
-                res[i] -= (d + 1.0 / theta) * np.log1p(-t_var[i])
-            else:
-                res[i] = -np.Inf
-    else:
-        res = np.sum(
-            np.log1p(
-                theta * np.linspace(start=1.0, stop=d - 1.0, num=int(d) - 1)
-            ),
-            axis=0
-        ) - (1.0 + theta) * lu - (d + 1.0 / theta) * np.log1p(t_var)
-    if is_log:
-        return res
-    else:
-        return np.exp(res)
 
 
 def diag_pdf_clayton(u_values, theta, is_log=False):
@@ -667,7 +672,7 @@ def dmle_copula_gumbel(u_values):
     ...    [0.51942063, 0.73040326, 0.25935125],
     ...    [0.46365886, 0.2459    , 0.83277053]
     ...    ]))
-    1.5138454093002933
+    1.0
     >>> dmle_copula_gumbel(np.array([
     ...   [0.42873569, 0.18285458, 0.9514195],
     ...   [0.25148149, 0.05617784, 0.3378213],
@@ -686,7 +691,7 @@ def dmle_copula_gumbel(u_values):
                 max_diag_pdf(
                     u_values=u_values,
                     diag_pdf=diag_pdf_gumbel,
-                    init=1.5,
+                    init=10,
                     bounds=(1.0 + float_info.min, 100.0)
                 )])
 
@@ -980,6 +985,7 @@ def eulerian_all(n):
            1.310354e+06, 1.310354e+06, 4.551920e+05, 4.784000e+04,
            1.013000e+03, 1.000000e+00])
     """
+    log.logging.info("computing eulerian all for %i ", n)
     return np.asarray(c_arch.eulerian_all(n))
 
 
@@ -1050,7 +1056,12 @@ def pdf_frank(u_values, theta, is_log=False):
     array([-1.88140742,  0.60959076, -3.6173953 , -1.88807634, -1.66399309,
             0.37026175, -2.40149211, -1.55091612,  0.08437906, -0.29091761])
     """
-    return c_arch.pdf_frank(u_values, theta, is_log)
+    return c_arch.pdf_frank(
+        u_values,
+        theta,
+        eulerian_all(u_values.shape[1] - 1),
+        is_log
+    )
 
 
 def ipsi_gumbel(u_values, theta, is_log=False):
