@@ -808,8 +808,9 @@ def process_bed(file_names: list,
                 score_cols: str = None,
                 pos_cols: list = None,
                 drop_unmatched: bool = True,
-                thread_num=mp.cpu_count(),
-                missing: float = None
+                cpu=mp.cpu_count(),
+                missing: float = None,
+                randomize_equality: bool = True
                 ):
     """
     Process a list of bed files names with the first names the merged bed files
@@ -824,7 +825,7 @@ def process_bed(file_names: list,
     :param score_cols: column name of the score to use
     :param pos_cols: list of position column name to sort and merge on
     :param drop_unmatched: bool
-    :param thread_num: int number of thread to use for merging
+    :param cpu: int number of thread to use for merging
     :param missing:
     :return: nothing
     """
@@ -838,7 +839,7 @@ def process_bed(file_names: list,
         score_cols=score_cols,
         pos_cols=pos_cols,
         drop_unmatched=drop_unmatched,
-        thread_num=thread_num
+        thread_num=cpu
     )
     if bed_files[0].shape[0] > 0:
         local_idr = idr_func(
@@ -850,7 +851,10 @@ def process_bed(file_names: list,
                 outdir=outdir
             ),
             threshold=threshold,
-            missing=missing
+            missing=missing,
+            cpu=cpu,
+            outdir=outdir,
+            randomize_equality=randomize_equality
         )
         writefiles(
             bed_files=bed_files,
