@@ -65,8 +65,10 @@ def process_matrix(file_name: str,
                    outdir: str,
                    idr_func: Callable[[np.array], np.array],
                    threshold: float = 0.0001,
-                   missing: float = None
-                   ):
+                   missing: float = None,
+                   cpu: int = 1,
+                   randomize_equality: bool = True
+    ):
     """
     Process a list of bed files names with the first names the merged bed files
     :param file_name: name of the matrix file
@@ -83,7 +85,10 @@ def process_matrix(file_name: str,
                 file_name=file_name
             ),
             threshold=threshold,
-            missing=missing
+            missing=missing,
+            cpu=cpu,
+            outdir=outdir,
+            randomize_equality=randomize_equality
         ),
         outdir=outdir
     )
