@@ -13,30 +13,84 @@ call for the merged replicate. This tool computes and appends a IDR column to
 NarrowPeaks files.
 """
 
+from scipy.optimize import minimize_scalar
 from scipy.optimize import minimize
-from scipy.special import logsumexp
+from scipy.stats import bernoulli
+from scipy.stats import multinomial
 import numpy as np
-import midr.log as log
-import midr.archimedean as archimedean
 import c_archimedean as c_arch
 from midr.auxiliary import compute_empirical_marginal_cdf, compute_rank
 from midr.idr import sim_m_samples
+import midr.log as log
+import midr.archimedean as archimedean
 import midr.archimediean_plots as archimediean_plots
+from pathlib import PurePath
+from copy import copy
+
+
+def random_u(n = 1000, dim = 5, name = "clayton", theta=15):
+    import rpy2
+    from rpy2.robjects.packages import importr
+    copula = importr("copula")
+    if name == "clayton":
+        u_values = copula.rCopula(
+            n=n,
+            copula=copula.claytonCopula(param=theta, dim=dim)
+        )
+        return np.array(u_values)
+    if name == "frank":
+        u_values = copula.rCopula(
+            n=n,
+            copula=copula.frankCopula(param=theta, dim=dim)
+        )
+        return np.array(u_values)
+    if name == "gumbel":
+        u_values = copula.rCopula(
+            n=n,
+            copula=copula.gumbelCopula(param=theta, dim=dim)
+        )
+        return np.array(u_values)
+    exit(-1)
+
+
+def mixture_archimedian(n, dim, params_list):
+    from scipy.stats import multinomial
+    l_state = multinomial.rvs(n=1, p=params_list['alpha'], size=n) == 1
+    score = np.zeros((n, dim))
+    for copula in copula_names():
+        score[l_state[:, params_list['order'][copula]], :] = random_u(
+            n=n,
+            dim=dim,
+            name=copula,
+            theta=params_list[copula]['theta']
+        )[l_state[:, params_list['order'][copula]], :]
+    return score
+
+
+def random_mix(n, dim, params_list):
+    k_state = bernoulli.rvs(p=params_list['pi'], size=n) == 1
+    scores = np.random.rand(n, dim)
+    scores[k_state, :] = mixture_archimedian(
+        n=n,
+        dim=dim,
+        params_list=params_list
+    )[k_state, :]
+    return {'X': scores, 'K': np.array(k_state, dtype="float64")}
 
 COPULA_DENSITY = {
+    'gumbel': c_arch.pdf_gumbel,
     'clayton': c_arch.pdf_clayton,
-    'frank': c_arch.pdf_frank,
-    'gumbel': c_arch.pdf_gumbel
+    'frank': archimedean.pdf_frank
 }
 DMLE_COPULA = {
     'clayton': archimedean.dmle_copula_clayton,
-    #'frank': archimedean.dmle_copula_frank,
-    'frank': lambda x: 2.0,
+    'frank': archimedean.dmle_copula_frank,
+    # 'frank': lambda x: 15.0,
     'gumbel': archimedean.dmle_copula_gumbel
 }
 
 
-def copula_density(copula, **kwargs):
+def copula_density(copula, u_values, theta, is_log=False):
     """
     return a copula density function
     :param copula:
@@ -44,6 +98,15 @@ def copula_density(copula, **kwargs):
     >>> u_values = np.random.rand(10000,30)
     >>> np.count_nonzero(
     ...     ~np.isnan(
+    ...         copula_density('clayton',
+    ...             u_values = u_values,
+    ...             theta = 6.0,
+    ...             is_log = True
+    ... )))
+    10000
+    >>> u_values = np.random.rand(10000,30)
+    >>> np.count_nonzero(
+    ...     ~np.isnan(
     ...         copula_density('frank',
     ...             u_values = u_values,
     ...             theta = 6.0,
@@ -52,8 +115,11 @@ def copula_density(copula, **kwargs):
     10000
     """
     return np.nan_to_num(
-        COPULA_DENSITY[copula](**kwargs),
-        copy=False
+        COPULA_DENSITY[copula](u_values, theta, is_log),
+        copy=False,
+        nan=0.0,
+        posinf=0.0,
+        neginf=0.0
     )
 
 
@@ -365,6 +431,21 @@ def expectation_k(u_values, params_list, dcopula=None):
     ...    }
     ... )))
     10000
+    >>> np.random.seed(123)
+    >>> u_values = np.random.rand(10,5)
+    >>> np.exp(expectation_k(u_values * u_values,
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
+    ...        'missing': None
+    ...    }
+    ... ))
+    array([0.05334154, 0.01773934, 0.22080577, 0.11382547, 0.43456438,
+           0.66661805, 0.507021  , 0.00266574, 0.02776513, 0.00915402])
     """
     if dcopula is None:
         dcopula = np.zeros((u_values.shape[0], copula_number()))
@@ -377,8 +458,13 @@ def expectation_k(u_values, params_list, dcopula=None):
                     params_list=params_list
                 )
     k_state = archimedean.lsum(dcopula, is_log=True, axis=1)
-    k_state += np.log(1 - np.exp(params_list['pi']))
-    return k_state - np.log(np.exp(params_list['pi']) + np.exp(k_state))
+    return (
+        k_state + np.log(1.0 - np.exp(params_list['pi'])) -
+        np.log(
+            np.exp(params_list['pi']) + (1.0 - np.exp(params_list['pi'])) *
+            np.exp(k_state)
+        )
+    )
 
 
 def expectation_l(u_values, params_list, dcopula=None):
@@ -684,6 +770,60 @@ def loglikelihood_theta(theta, u_values, copula, params_list, dcopula=None):
     ...    }
     ... )
     1.4072509026421054
+    >>> loglikelihood_theta(
+    ...    theta=2,
+    ...    u_values = np.array([
+    ...       [0.42873569, 0.18285458, 0.9514195],
+    ...       [0.25148149, 0.05617784, 0.3378213],
+    ...       [0.79410993, 0.76175687, 0.0709562],
+    ...       [0.02694249, 0.45788802, 0.6299574],
+    ...       [0.39522060, 0.02189511, 0.6332237],
+    ...       [0.0, 0.38075101, 0.5185625],
+    ...       [0.66878367, 0.38075101, 0.5185625],
+    ...       [0.90365653, 0.19654621, 0.6809525],
+    ...       [0.28607729, 0.82713755, 0.7686878],
+    ...       [0.22437343, 0.16907646, 0.5740400],
+    ...       [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    copula = "clayton",
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6, 'pi': 0.2},
+    ...        'gumbel': {'theta': 6, 'pi': 0.2},
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': 0.0
+    ...    }
+    ... )
+    1.3944790628903807
+    >>> loglikelihood_theta(
+    ...    theta=2,
+    ...    u_values = np.array([
+    ...       [0.42873569, 0.18285458, 0.9514195],
+    ...       [0.25148149, 0.05617784, 0.3378213],
+    ...       [0.79410993, 0.76175687, 0.0709562],
+    ...       [0.02694249, 0.45788802, 0.6299574],
+    ...       [0.39522060, 0.02189511, 0.6332237],
+    ...       [0.0, 0.38075101, 0.5185625],
+    ...       [0.66878367, 0.38075101, 0.5185625],
+    ...       [0.90365653, 0.19654621, 0.6809525],
+    ...       [0.28607729, 0.82713755, 0.7686878],
+    ...       [0.22437343, 0.16907646, 0.5740400],
+    ...       [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    copula = "gumbel",
+    ...    params_list={
+    ...        'pi': np.log(0.8),
+    ...        'alpha': np.log([0.3333, 0.3333, 0.3333]),
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'frank': {'theta': 6, 'pi': 0.2},
+    ...        'gumbel': {'theta': 6, 'pi': 0.2},
+    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'missing': 0.0
+    ...    }
+    ... )
+    1.2126174989266756
     """
     if dcopula is None:
         dcopula = partial_loglikelihood_theta(
@@ -831,18 +971,30 @@ def minimize_pi(k_state):
     find theta that minimize the likelihood of the copula density
     :param k_state:
     :return:
+    >>> THETA_TEST = {'pi': 0.35, 'mu': 2.0, 'sigma': 1.0, 'rho': 1.0}
+    >>> THETA_0 = {'mu': 0.0, 'sigma': 1.0, 'rho': 0.0}
+    >>> np.random.seed(123)
+    >>> DATA = sim_m_samples(n_value=10000,
+    ...                      m_sample=2,
+    ...                      theta_0=THETA_0,
+    ...                      theta_1=THETA_TEST)
+    >>> np.exp(minimize_pi(np.log(DATA['K'])))
+    0.3488999999999998
+    >>> np.random.seed(123)
     >>> np.exp(minimize_pi(
-    ...     k_state = np.log(np.array([0.99275263, 0.87291238, 0.99918233,
-    ...                         0.99339966, 0.99138885, 0.81964467,
-    ...                         0.99643828, 0.99003915,
-    ...                         0.91222219, 0.94832496]))))
-    0.9516305099999998
+    ...     k_state = np.log(np.random.rand(10000, 1))))
+    0.4982311453103447
     """
-    return archimedean.lsum(k_state, is_log=True, axis=0)[0] - float(
-        np.log(len(k_state)))
+    return min([
+        archimedean.lsum(k_state, is_log=True, axis=0)[0] - float(
+        np.log(len(k_state))),
+        np.log(1.0 - 1e-14)
+        ]
+    )
 
 
-def minimize_theta(u_values, copula, params_list, dcopula=None):
+def minimize_theta(u_values, copula, params_list, dcopula=None,
+                   threshold=1e-10, optim=None, outdir="log"):
     """
     find theta that minimize the likelihood of the copula density
     :param u_values:
@@ -850,227 +1002,147 @@ def minimize_theta(u_values, copula, params_list, dcopula=None):
     :param params_list:
     :param dcopula:
     :return:
-    >>> # data generated with theta = 5
-    >>> minimize_theta(np.array([
-    ...    [0.2104386, 0.6909995, 0.2195247],
-    ...    [0.9185531, 0.9172150, 0.9201070],
-    ...    [0.2869368, 0.3535080, 0.2931358],
-    ...    [0.3124657, 0.2484218, 0.0519745],
-    ...    [0.6768665, 0.6989673, 0.5448870],
-    ...    [0.5302475, 0.9583231, 0.8025312],
-    ...    [0.7562460, 0.9274616, 0.6933584],
-    ...    [0.3130325, 0.3607322, 0.5237398],
-    ...    [0.4842922, 0.3572355, 0.3118641],
-    ...    [0.5733899, 0.5767076, 0.1369106],
-    ...    ]),
-    ...    copula="frank",
-    ...    params_list={
+    >>> import rpy2
+    >>> from rpy2.robjects.packages import importr
+    >>> copula = base = importr("copula")
+    >>> u_values = copula.rCopula(
+    ...    n=1000,
+    ...    copula=copula.claytonCopula(param=15, dim = 5)
+    ... )
+    >>> np.round(minimize_theta(
+    ...     np.array(u_values),
+    ...     copula="clayton",
+    ...     params_list={
     ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
+    ...        'alpha': np.log([0.0, 1., 0.]),
     ...        'order': {'frank': 2, 'gumbel': 0, 'clayton': 1},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
     ...        'missing': None
     ...    }
+    ... ))
+    15.0
+    >>> u_values = copula.rCopula(
+    ...    n=1000,
+    ...    copula=copula.frankCopula(param=12, dim = 5)
     ... )
-    5.4438915807028305
-    >>> # data generated with theta = 5 and 10% missing
-    >>> minimize_theta(np.array([
-    ...    [0.2104386, 0.6909995, 0.2195247],
-    ...    [0.9185531, 0.0000000, 0.9201070],
-    ...    [0.2869368, 0.3535080, 0.2931358],
-    ...    [0.3124657, 0.2484218, 0.0519745],
-    ...    [0.0000000, 0.6989673, 0.5448870],
-    ...    [0.5302475, 0.9583231, 0.0000000],
-    ...    [0.7562460, 0.9274616, 0.6933584],
-    ...    [0.3130325, 0.3607322, 0.5237398],
-    ...    [0.4842922, 0.3572355, 0.3118641],
-    ...    [0.5733899, 0.5767076, 0.1369106],
-    ...    ]),
-    ...    copula="frank",
-    ...    params_list={
+    >>> np.round(minimize_theta(
+    ...     np.array(u_values),
+    ...     copula="frank",
+    ...     params_list={
     ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
+    ...        'alpha': np.log([0., 0., 1.]),
     ...        'order': {'frank': 2, 'gumbel': 0, 'clayton': 1},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
-    ...        'missing': 0.0
-    ...    }
-    ... )
-    4.979949296180444
-    >>> # data generated with theta = 5
-    >>> minimize_theta(np.array([
-    ...    [0.24078826, 0.37870827, 0.16061743],
-    ...    [0.25736139, 0.18174993, 0.38443831],
-    ...    [0.16239852, 0.21764999, 0.23479306],
-    ...    [0.22269799, 0.24225874, 0.21488540],
-    ...    [0.32328339, 0.53500332, 0.31741874],
-    ...    [0.07236942, 0.16006716, 0.08322168],
-    ...    [0.01503262, 0.02696656, 0.07374267],
-    ...    [0.14510121, 0.13951564, 0.20262086],
-    ...    [0.03519165, 0.14688405, 0.00565128],
-    ...    [0.58870457, 0.68505697, 0.58995495]
-    ...    ]),
-    ...    copula="gumbel",
-    ...    params_list={
-    ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
-    ...        'order': {'frank': 0, 'gumbel': 2, 'clayton': 1},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
     ...        'missing': None
     ...    }
+    ... ))
+    12.0
+    >>> u_values = copula.rCopula(
+    ...    n=10000,
+    ...    copula=copula.gumbelCopula(param=17, dim = 5)
     ... )
-    4.5616032335264665
-    >>> # data generated with theta = 5 and 10% missing
-    >>> minimize_theta(np.array([
-    ...    [0.24078826, 0.37870827, 0.16061743],
-    ...    [0.25736139, 0.18174993, 0.38443831],
-    ...    [0.16239852, 0.21764999, 0.00000000],
-    ...    [0.22269799, 0.24225874, 0.21488540],
-    ...    [0.32328339, 0.53500332, 0.31741874],
-    ...    [0.07236942, 0.16006716, 0.00000000],
-    ...    [0.01503262, 0.02696656, 0.07374267],
-    ...    [0.00000000, 0.13951564, 0.20262086],
-    ...    [0.03519165, 0.14688405, 0.00565128],
-    ...    [0.58870457, 0.68505697, 0.58995495]
-    ...    ]),
-    ...    copula="gumbel",
-    ...    params_list={
+    >>> np.round(minimize_theta(
+    ...     np.array(u_values),
+    ...     copula="gumbel",
+    ...     params_list={
     ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
-    ...        'order': {'frank': 0, 'gumbel': 2, 'clayton': 1},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
-    ...        'missing': 0.0
-    ...    }
-    ... )
-    4.106380435452723
-    >>> # data generated with theta = 5
-    >>> minimize_theta(np.array([
-    ...    [0.10112599, 0.13290793, 0.180375718],
-    ...    [0.91367700, 0.89759850, 0.805749718],
-    ...    [0.58878457, 0.91666470, 0.661574429],
-    ...    [0.90617569, 0.92697520, 0.907013741],
-    ...    [0.70750326, 0.51343695, 0.519591735],
-    ...    [0.32777265, 0.24957232, 0.427083826],
-    ...    [0.01446402, 0.01232720, 0.009718799],
-    ...    [0.31459652, 0.24885640, 0.315366745],
-    ...    [0.09053508, 0.06687791, 0.075200268],
-    ...    [0.46313275, 0.59155993, 0.573731314],
-    ...    ]),
-    ...    copula="clayton",
-    ...    params_list={
-    ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
-    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
+    ...        'alpha': np.log([1., 0., 0.]),
+    ...        'order': {'frank': 2, 'gumbel': 0, 'clayton': 1},
+    ...        'frank': {'theta': 6},
+    ...        'gumbel': {'theta': 6},
+    ...        'clayton': {'theta': 6},
     ...        'missing': None
     ...    }
-    ... )
-    5.340780997857822
-    >>> # data generated with theta = 5 and 10% missing data
-    >>> minimize_theta(np.array([
-    ...    [0.00000000, 0.13290793, 0.180375718],
-    ...    [0.91367700, 0.89759850, 0.805749718],
-    ...    [0.58878457, 0.91666470, 0.00000000],
-    ...    [0.90617569, 0.92697520, 0.907013741],
-    ...    [0.00000000, 0.51343695, 0.519591735],
-    ...    [0.32777265, 0.24957232, 0.427083826],
-    ...    [0.01446402, 0.01232720, 0.009718799],
-    ...    [0.31459652, 0.24885640, 0.315366745],
-    ...    [0.09053508, 0.06687791, 0.075200268],
-    ...    [0.46313275, 0.59155993, 0.573731314],
-    ...    ]),
-    ...    copula="clayton",
-    ...    params_list={
-    ...        'pi': np.log(0.01),
-    ...        'alpha': np.log([0.005, 0.005, 0.99]),
-    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
-    ...        'frank': {'theta': 6, 'pi': 0.2},
-    ...        'gumbel': {'theta': 6, 'pi': 0.2},
-    ...        'clayton': {'theta': 6, 'pi': 0.2},
-    ...        'missing': 0.0
-    ...    }
-    ... )
-    5.677481753916402
+    ... ))
+    17.0
     """
-    log.logging.debug("%s", copula + " minimize_theta")
+    log.logging.info(
+        "minimize theta for %s",
+        copula
+    )
     old_theta = params_list[copula]['theta']
     log.logging.debug("%s", str([build_bounds(copula)]) + " minimize() bounds")
     log.logging.debug("%s", str(old_theta) + " old_theta")
-    if log.logging.root.level == log.logging.DEBUG:
-        archimediean_plots.pdf_copula_plot(
-            lower=build_bounds(copula)[0],
-            upper=build_bounds(copula)[1],
-            copula=copula,
-            pdf_function=loglikelihood_theta,
-            u_values=u_values,
-            params_list=params_list,
-        )
     if dcopula is None:
         dcopula = partial_loglikelihood_theta(
             u_values=u_values,
             params_list=params_list
         )
-    res = minimize(
-        fun=loglikelihood_theta,
-        args=(u_values, copula, params_list, dcopula),
-        x0=old_theta,
-        bounds=[build_bounds(copula)],
-    )
+    if optim is None:
+        res = minimize(
+            fun=loglikelihood_theta,
+            args=(u_values, copula, params_list, dcopula),
+            x0=old_theta,
+            bounds=[build_bounds(copula)],
+        )
+        new_theta = res.x[0]
+    else:
+        if optim == "SLSQP":
+            res = minimize(
+                fun=loglikelihood_theta,
+                x0=old_theta,
+                args=(u_values, copula, params_list, dcopula),
+                bounds=[build_bounds(copula)],
+                method="SLSQP",
+                tol=threshold
+            )
+            new_theta = res.x[0]
+        else:
+            np.seterr(over="ignore", invalid="ignore")
+            res = minimize_scalar(
+                fun=loglikelihood_theta,
+                args=(u_values, copula, params_list, dcopula),
+                bracket=build_bounds(copula),
+                bounds=build_bounds(copula),
+                method="brent",
+                options={"xtol": threshold, "maxiter": 1000}
+            )
+            new_theta = res.x
+    # if log.logging.root.level == log.logging.DEBUG:
+    #     archimediean_plots.pdf_copula_plot(
+    #         lower=build_bounds(copula)[0],
+    #         upper=build_bounds(copula)[1],
+    #         copula=copula,
+    #         pdf_function=loglikelihood_theta,
+    #         u_values=u_values,
+    #         params_list=params_list,
+    #         min_y=res.x,
+    #         outdir=outdir
+    #     )
     log.logging.debug("%s", res)
-    if np.isnan(res.x[0]):
+    if np.isnan(new_theta):
         log.logging.debug("%s", str(old_theta) + " new_theta = old_theta")
         return old_theta
     else:
-        log.logging.debug("%s", str(res.x[0]) + " new_theta")
-        return res.x[0]
+        log.logging.debug("%s", str(new_theta) + " new_theta")
+        return min(
+            max(new_theta, build_bounds(copula)[0]),
+            min(new_theta, build_bounds(copula)[1])
+        )
 
 
-def samic(x_score, threshold=1e-4, missing=None):
+def samic_init(u_values, threshold=None, missing=None, outdir=None):
     """
-    implementation of the samic method for m samples
-    :param x_score np.array of score (measures x samples)
-    :param threshold float min delta between every parameters between two
-    iterations
-    :param missing: float value equal to the value considered to be missing
-    :return (theta: dict, lidr: list) with theta the model parameters and
-    lidr the local idr values for each measures
-    >>> THETA_TEST_0 = {'mu': 0.0, 'sigma': 1.0, 'rho': 0.0}
-    >>> THETA_TEST_1 = {'pi': 0.1, 'mu': 4.0, 'sigma': 3.0, 'rho': 0.75}
-    >>> DATA = sim_m_samples(n_value=10000,
-    ...                      m_sample=20,
-    ...                      theta_0=THETA_TEST_0,
-    ...                      theta_1=THETA_TEST_1)
-    >>> lidr = samic(DATA["X"], threshold=0.0001)
-    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr))  <= 0.11
-    True
+    initialisation function for samic procedure
+    :return:
     """
-    log.logging.info("%s", "computing idr")
-    u_values = compute_empirical_marginal_cdf(
-        compute_rank(
-            x_score=x_score,
-            missing=missing
-        ),
-        gaussian=True,
-        missing=missing
-    )
     params_list = {'order': {}}
     i = 0
     for copula in copula_names():
         params_list[copula] = {
-            'theta': 4.0,
-            'theta_old': np.nan,
+            'theta': 10.,
+            'theta_old': np.nan
         }
         params_list['order'][copula] = i
         i += 1
+    if threshold is None:
+        params_list['threshold'] = 10**-(len(str(u_values.shape[0])) - 1)
+    else:
+        params_list['threshold'] = threshold
     params_list['missing'] = missing
     params_list['k_state'] = np.zeros(u_values.shape[0])
     params_list['l_state'] = np.zeros((u_values.shape[0], copula_number()))
@@ -1080,55 +1152,244 @@ def samic(x_score, threshold=1e-4, missing=None):
         np.repeat(1.0 / copula_number(), copula_number())
     )
     params_list['alpha_old'] = params_list['alpha'][:]
-    dcopula = partial_loglikelihood_theta(
+    params_list['dcopula'] = partial_loglikelihood_theta(
         u_values=u_values,
-        params_list=params_list
+        params_list=params_list,
     )
-    while delta(params_list, threshold):
-        params_list['k_state'] = expectation_k(
+    params_list['outdir'] = outdir
+    params_list['log_file'] = write_header(params_list)
+    log.logging.info(
+        "%s",
+        log_samic(params_list)
+    )
+    return params_list
+
+
+def samic_expectation(u_values, params_list):
+    """
+    exceptation step of the EM
+    :return:
+    """
+    params_list['k_state'] = expectation_k(
+        u_values=u_values,
+        params_list=params_list,
+        dcopula=params_list['dcopula'],
+    )
+    params_list['l_state'] = expectation_l(
+        u_values=u_values,
+        params_list=params_list,
+        dcopula=params_list['dcopula'],
+    )
+    params_list['alpha_old'] = params_list['alpha'][:]
+    params_list['pi_old'] = params_list['pi']
+    return params_list
+
+
+def samic_minimize(u_values, params_list, outdir):
+    """
+    minimization step of the EM
+    :param outdir:
+    :param u_values:
+    :param params_list:
+    :return:
+    """
+    params_list['pi'] = minimize_pi(
+        k_state=params_list['k_state']
+    )
+    params_list['alpha'] = minimize_alpha(
+        l_state=params_list['l_state']
+    )
+    for copula in COPULA_DENSITY:
+        params_list[copula]['theta_old'] = params_list[copula]['theta']
+        params_list[copula]['theta'] = minimize_theta(
             u_values=u_values,
+            copula=copula,
             params_list=params_list,
-            dcopula=dcopula
+            dcopula=params_list['dcopula'],
+            threshold=params_list['threshold'],
+            outdir=outdir
         )
-        params_list['l_state'] = expectation_l(
-            u_values=u_values,
-            params_list=params_list,
-            dcopula=dcopula
-        )
-        params_list['alpha_old'] = params_list['alpha'][:]
-        params_list['pi_old'] = params_list['pi']
-        params_list['pi'] = minimize_pi(
-            k_state=params_list['k_state']
-        )
-        params_list['alpha'] = minimize_alpha(
-            l_state=params_list['l_state']
-        )
-        log.logging.info(
-            "%s",
-            log_samic(params_list)
-        )
-        for copula in COPULA_DENSITY:
-            params_list[copula]['theta_old'] = params_list[copula]['theta']
-            params_list[copula]['theta'] = minimize_theta(
+        params_list['dcopula'][:, params_list['order'][copula]] = \
+            partial_loglikelihood_copula(
                 u_values=u_values,
                 copula=copula,
                 params_list=params_list,
-                dcopula=dcopula
             )
-            dcopula[:, params_list['order'][copula]] = \
-                partial_loglikelihood_copula(
+        log.logging.info(
+            "%s %s",
+            copula,
+            log_samic(params_list)
+        )
+    return params_list
+
+
+def variational_samic_minimize(u_values, params_list, outdir, frac=.8):
+    """
+    minimization step of the EM
+    :param outdir:
+    :param u_values:
+    :param params_list:
+    :return:
+    """
+    sample = bernoulli.rvs(p=frac, size=u_values.shape[0]) == 1
+    run_params_list = copy(params_list)
+    run_params_list['dcopula'] = run_params_list['dcopula'][sample, :]
+    run_params_list['k_state'] = run_params_list['k_state'][sample]
+    run_params_list['l_state'] = run_params_list['l_state'][sample, :]
+    run_params_list = samic_minimize(
+        u_values=u_values[sample, :],
+        params_list=run_params_list,
+        outdir=outdir
+    )
+    params_list['pi'] = run_params_list['pi']
+    params_list['alpha'] = run_params_list['alpha']
+    params_list['clayton']['theta'] = run_params_list['clayton']['theta']
+    params_list['frank']['theta'] = run_params_list['frank']['theta']
+    params_list['gumbel']['theta'] = run_params_list['gumbel']['theta']
+    for copula in COPULA_DENSITY:
+        params_list['dcopula'][:, params_list['order'][copula]] = \
+            partial_loglikelihood_copula(
                 u_values=u_values,
                 copula=copula,
-                params_list=params_list
-            )
-            log.logging.info(
-                "%s %s",
-                copula,
-                log_samic(params_list)
+                params_list=params_list,
             )
+    return params_list
+
+
+def samic_optim(u_values, threshold=None, missing=None, outdir=None):
+    """
+    run EM to optimize model parameters
+    :return:
+    >>> u_values = random_mix(n=1000, dim=5, params_list={
+    ...        'pi': 0.6,
+    ...        'alpha': [0.4, 0.2, 0.4],
+    ...        'order': {'frank': 0, 'gumbel': 1, 'clayton': 2},
+    ...        'clayton': {'theta': 6},
+    ...        'frank': {'theta': 15},
+    ...        'gumbel': {'theta': 4},
+    ...        'missing': None
+    ...    })
+    >>> results = samic_optim(u_values['X'], threshold=0.01)
+    >>> np.exp(results['pi'])
+    >>> results['order']
+    >>> np.exp(results['alpha'])
+    >>> results['gumbel']['theta']
+    >>> results['frank']['theta']
+    >>> results['clayton']['theta']
+    >>> def theta_clayton(theta):
+    ...     return samic_optim(random_u(100, 5, "clayton", np.float(theta)))[
+    ...     'clayton']['theta']
+    >>> def theta_gumbel(theta):
+    ...     return samic_optim(random_u(100, 5, "gumbel", np.float(theta)))[
+    ...     'gumbel']['theta']
+    >>> def theta_frank(theta):
+    ...     return samic_optim(random_u(100, 5, "frank", np.float(theta)))[
+    ...     'frank']['theta']
+    >>> np.mean(np.array(list(map(theta_clayton, np.linspace(2., 40., 40)))
+    ... ) - np.linspace(2., 40., 40)) < 1.
+    True
+    >>> np.mean(np.array(list(map(theta_gumbel, np.linspace(2., 40., 40)))
+    ... ) - np.linspace(2., 40., 40)) < 1.
+    True
+    >>> np.mean(np.array(list(map(theta_frank, np.linspace(2., 40., 10)))
+    ... ) - np.linspace(2., 40., 10)) < 1.
+    True
+    >>> theta = samic_optim(random_u(1000, 5, "clayton", 15))['clayton']['theta']
+    >>> np.round(theta)
+    15.0
+    >>> theta = samic_optim(random_u(1000, 5, "frank", 15))['frank']['theta']
+    >>> np.round(theta)
+    17.0
+    >>> theta = samic_optim(random_u(1000, 5, "gumbel", 15))['gumbel']['theta']
+    >>> np.round(theta)
+    15.0
+    """
+    params_list = samic_init(
+        u_values=u_values,
+        threshold=threshold,
+        missing=missing,
+        outdir=outdir
+    )
+    if log.logging.root.level == log.logging.DEBUG:
+        params_plot = archimediean_plots.ParamsPlot(
+            params_list=params_list,
+            copula_names=copula_names(),
+            outdir=outdir
+        )
+    while delta(params_list, params_list['threshold']):
+        params_list = samic_expectation(
+            u_values=u_values,
+            params_list=params_list
+        )
+        params_list = samic_minimize(
+            u_values=u_values,
+            params_list=params_list,
+            outdir=outdir
+        )
+        log.logging.warning(
+            "%s",
+            log_samic(params_list)
+        )
+        if log.logging.root.level == log.logging.DEBUG:
+            # archimediean_plots.pdf_density_copula_plot(
+            #      pdf_function=censored_copula_density,
+            #      params_list=params_list,
+            #      copula_names=copula_names(),
+            #      u_values=u_values,
+            #      outdir=outdir
+            # )
+            params_plot.update(params_list=params_list)
+    
+    if log.logging.root.level == log.logging.DEBUG:
+        params_plot.plot()
+    return params_list
+
+
+def samic(x_score, threshold=None, missing=None, cpu=1, outdir=None,
+          randomize_equality=True):
+    """
+    implementation of the samic method for m samples
+    :param cpu:
+    :param x_score np.array of score (measures x samples)
+    :param threshold float min delta between every parameters between two
+    iterations
+    :param missing: float value equal to the value considered to be missing
+    :return (theta: dict, lidr: list) with theta the model parameters and
+    lidr the local idr values for each measures
+    >>> THETA_TEST_0 = {'mu': 0.0, 'sigma': 1.0, 'rho': 0.0}
+    >>> THETA_TEST_1 = {'pi': 0.3, 'mu': 2.0, 'sigma': 2.0, 'rho': 0.65}
+    >>> np.random.seed(123)
+    >>> DATA = sim_m_samples(n_value=1000,
+    ...                      m_sample=4,
+    ...                      theta_0=THETA_TEST_0,
+    ...                      theta_1=THETA_TEST_1)
+    >>> lidr = samic(DATA["X"])
+    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr))
+    0.714
+    >>> (np.sum((lidr < 0.5).all() == DATA["K"]) / len(lidr)) - ( 1. -
+    ... THETA_TEST_1['pi']) <= 0.1
+    True
+    """
+    log.logging.info("%s", "computing idr")
+    u_values = compute_empirical_marginal_cdf(
+        compute_rank(
+            x_score=x_score,
+            missing=missing,
+            randomize_equality=randomize_equality
+        ),
+        missing=missing,
+    )
+    params_list = samic_optim(
+        u_values=u_values,
+        threshold=threshold,
+        missing=missing,
+        outdir=outdir
+    )
+    if params_list['outdir'] is not None:
+        params_list['log_file'].close()
     return local_idr(
         u_values=u_values,
-        params_list=params_list
+        params_list=params_list,
     )
 
 
@@ -1138,6 +1399,7 @@ def log_samic(params_list):
     :param params_list:
     :return:
     """
+    write_params(params_list)
     log_str = str('{' +
                   '"pi": ' + str(np.exp(params_list['pi']))
                   )
@@ -1146,12 +1408,52 @@ def log_samic(params_list):
             ', \n"' + copula + '": {' +
             '"alpha": ' +
             str(np.exp(params_list['alpha'][params_list["order"][copula]])) +
-            ', ' '"theta": ' +
+            ', "theta": ' +
             str(params_list[copula]['theta']) +
             '}')
     return log_str + ' }'
 
 
+def write_header(params_list):
+    """
+    write parameter estimation to log_file
+    :param params_list:
+    :return:
+    """
+    log_file = None
+    if params_list['outdir'] is not None:
+        log_file = open(
+            PurePath(params_list['outdir']).joinpath("idr_params.tsv"),
+            "w"
+        )
+        log_str = "pi"
+        for copula in copula_names():
+            log_str += "\t" + "alpha_" + copula
+        for copula in copula_names():
+            log_str += "\t" + "theta_" + copula
+        log_file.write(log_str + "\n")
+    return log_file
+
+
+def write_params(params_list):
+    """
+    write parameter estimation to log_file
+    :param params_list:
+    :return:
+    """
+    log_str = str(np.exp(params_list['pi']))
+    for copula in copula_names():
+        log_str += "\t" + str(
+            np.exp(params_list['alpha'][params_list["order"][copula]])
+        )
+    for copula in copula_names():
+        log_str += "\t" + str(
+            params_list[copula]['theta']
+        )
+    if params_list['outdir'] is not None:
+        params_list['log_file'].write(log_str + "\n")
+
+
 if __name__ == "__main__":
     import doctest
 
